package org.xbib.interlibrary.client.k10plus;

import org.xbib.interlibrary.action.avail.AvailService;
import org.xbib.interlibrary.action.avail.AvailServiceProvider;
import org.xbib.interlibrary.action.resolve.ResolveService;
import org.xbib.interlibrary.action.resolve.ResolveServiceProvider;
import org.xbib.interlibrary.action.search.SearchService;
import org.xbib.interlibrary.action.search.SearchServiceProvider;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.client.k10plus.sru.SRUAvailService;
import org.xbib.interlibrary.client.k10plus.sru.SRUResolveService;
import org.xbib.interlibrary.client.k10plus.sru.SRUSearchService;
import org.xbib.interlibrary.client.k10plus.z.ZAvailService;
import org.xbib.interlibrary.client.k10plus.z.ZResolveService;
import org.xbib.interlibrary.client.k10plus.z.ZSearchService;
import org.xbib.interlibrary.sru.SRUClientHelper;

public class Provider {

    volatile static SRUClientHelper clientHelper;

    public Provider() {
    }

    public static class SRUResolveServiceProvider implements ResolveServiceProvider {

        private SRUResolveService resolveService;

        public SRUResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (resolveService == null) {
                resolveService = new SRUResolveService(arguments, clientHelper);
            }
            return resolveService;
        }
    }

    public static class SRUAvailServiceProvider implements AvailServiceProvider {

        private SRUAvailService availService;

        public SRUAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (availService == null) {
                availService = new SRUAvailService(arguments, clientHelper);
            }
            return availService;
        }
    }

    public static class SRUSearchServiceProvider implements SearchServiceProvider {

        private SRUSearchService searchService;

        public SRUSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (searchService == null) {
                searchService = new SRUSearchService(arguments, clientHelper);
            }
            return searchService;
        }
    }


    public static class ZAvailServiceProvider implements AvailServiceProvider {

        private ZAvailService availService;

        public ZAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (availService == null) {
                availService = new ZAvailService(arguments);
            }
            return availService;
        }
    }

    public static class ZResolveServiceProvider implements ResolveServiceProvider {

        private ZResolveService resolveService;

        public ZResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (resolveService == null) {
                resolveService = new ZResolveService(arguments);
            }
            return resolveService;
        }
    }

    public static class ZSearchServiceProvider implements SearchServiceProvider {

        private ZSearchService searchService;

        public ZSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (searchService == null) {
                searchService = new ZSearchService(arguments);
            }
            return searchService;
        }
    }
}
