package org.xbib.interlibrary.client.k10plus.z;

import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.z.AbstractZAvailService;
import org.xbib.interlibrary.z.DisplayHelper;
import org.xbib.marc.MarcRecord;

import java.util.Locale;

public class ZAvailService extends AbstractZAvailService {

    public ZAvailService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source.toLowerCase(Locale.ROOT).contains("k10plus") ? "K10PLUS" : null;
    }

    @Override
    protected void avail(AvailRequest availRequest,
                         AvailServiceResponseBuilder availResponse,
                         MultiMap<String, LibraryService> multiMap,
                         MarcRecord marcRecord) {
        boolean found = false;
        if (availRequest.isBibliographicDescriptionEnabled()) {
            availResponse.setBibliographicDescription(marcRecord);
            TinyMap.Builder<String, Object> builder = TinyMap.builder();
            DisplayHelper.display(builder, marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                    marcHelper.getRecordIdSupplier(marcRecord));
            availResponse.setDisplay(builder.build());
        }
        for (LibraryService service : extractGermanInterlibraryServices(availRequest, marcRecord)) {
            multiMap.put(service.getISIL(), service);
            found = true;
        }
        if (found) {
            availResponse.setStatus(getName(), Response.Status.OK);
        } else {
            availResponse.setStatus(getName(), Response.Status.EMPTY_RESPONSE);
        }
    }

    @Override
    protected String normalizeSourceId(String sourceId) {
        return sourceId;
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
